@@ -6,13 +6,13 @@ Rubric::WebApp - the web interface to Rubric
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
- $Id: WebApp.pm,v 1.55 2004/12/18 03:05:08 rjbs Exp $
+ $Id: WebApp.pm,v 1.69 2005/01/15 03:53:50 rjbs Exp $
 
 =cut
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -46,7 +46,6 @@ basic dispatch table looks something like this:
  /post        | post or edit an entry (must be logged in) | post
  /edit        | edit an entry (must be logged in)         | edit
  /delete      | delete an entry (must be logged in)       | delete
- /recent      | see recent entries                        | recent
  /user/NAME   | see a user's entries                      | user
  /user/N/TAGS | see a user's entries for given tags       | user
  /tag/TAGS    | see entries for given tags                | tag
@@ -54,12 +53,13 @@ basic dispatch table looks something like this:
 
 If the system is private and no user is logged in, the default action is to
 display a login screen.  If the system is public, or a user is logged in, the
-default action is to display recent entries.
+default action is to display entries.
 
 =cut
 
 use base qw(CGI::Application);
 use CGI::Application::Session;
+use CGI::Carp qw(fatalsToBrowser);
 
 use Digest::MD5 qw(md5_hex);
 
@@ -71,14 +71,9 @@ use Email::Send;
 
 use Rubric::Config;
 use Rubric::Entry;
-use Rubric::Link;
 use Rubric::Renderer;
-use Rubric::User;
 use Rubric::WebApp::URI;
 
-use Template;
-use URI; 
-
 =head1 METHODS
 
 =head2 redirect($uri, $message)
@@ -120,60 +115,20 @@ sub cgiapp_init {
 	my ($self) = @_;
 
 	$self->session_config( COOKIE_PARAMS => { -expires => '+7d' } );
-	$self->check_for_login;
-	$self->get_current_user;
+	
+	my $login_class = Rubric::Config->login_class;
+	eval "require $login_class";
+	$login_class->check_for_login($self);
+
 	$self->check_pager_data;
 
 	my @path = split '/', $self->query->path_info;
-	shift @path for (1 .. 2);
-	$self->param(path => \@path);
+	$self->param(path => [ @path[ 2 .. $#path ] ]);
 }
 
-=head2 check_for_login
-
-This method is called by C<cgiapp_init>, and checks for a login attempt in the
-submitted request.  A login request is made with two CGI parameters:
-
- user     - the user's username
- password - the user's password
-
-If these match, the user is now logged in.  If not, it is ignored.
-
-However!  If the REMOTE_USER environment variable is set, because the user has
-logged in via HTTP, the value of REMOTE_USER is used as the current user.
-
-=cut
-
-sub check_for_login {
+sub next_path_part {
 	my ($self) = @_;
-
-	return $self->session->param('current_user', $ENV{REMOTE_USER})
-		if ($ENV{REMOTE_USER});
-
-	return unless my $userid = $self->query->param('user');
-	return unless my $login_user = Rubric::User->retrieve($userid);
-
-	my $login_pass = $self->query->param('password');
-
-	if (md5_hex($login_pass) eq $login_user->password) {
-		$login_user->verification_code
-			? $self->param('user_pending', 1)
-			: $self->session->param('current_user',$self->query->param('user'))
-	}
-}
-
-=head2 get_current_user
-
-This method is called by C<cgiapp_init>, and retrieves the Rubric::User object
-for the currently logged-in user, if any.
-
-=cut
-
-sub get_current_user {
-	my ($self) = @_;
-
-	return unless my $userid = $self->session->param('current_user');
-	$self->param(current_user => Rubric::User->retrieve($userid));
+	my $username = shift @{$self->param('path')};
 }
 
 =head2 check_pager_data
@@ -223,8 +178,8 @@ sub template {
 	$stash->{per_page} = $self->param('per_page');
 	$stash->{page} = $self->param('page');
 
-	my $type = $self->query->param('format') || 'html';
-	$type = 'html' unless $type =~ /^\w+$/;
+	my $type = $self->query->param('format');
+	   $type = 'html' unless $type and $type =~ /^\w+$/;
 
 	my ($content_type, $output) =
 		Rubric::Renderer->process($template, $type, $stash);
@@ -249,15 +204,31 @@ sub setup {
 	$self->run_modes([ qw(doc login newuser verify) ]);
 
 	if ($self->param('current_user') or not Rubric::Config->private_system) {
-		$self->start_mode('recent');
+		$self->start_mode('entries');
 		$self->run_modes([
-			qw(delete edit entry link logout post preferences recent user tag)
+			qw(delete edit entries entry link logout post preferences)
 		]);
 	}
 
 	$self->run_modes(AUTOLOAD => 'redirect_root');
 }
 
+=head2 entries
+
+This passes off responsibility to the class named in the C<entries_query_class>
+configuration option.  This option defaults to Rubric::WebApp::EntriesQuery.
+
+=cut
+
+sub entries {
+	my ($self) = @_;
+
+	my $entries_class = Rubric::Config->entries_query_class;
+	eval "require $entries_class";
+	if ($@) { warn $@; return $self->redirect_root; }
+	$entries_class->entries($self);
+}
+
 =head2 entry
 
 This displays the single requested entry.
@@ -286,11 +257,8 @@ id, and puts the corresponding entry in the "entry" parameter.
 sub get_entry {
 	my ($self) = @_;
 
-	my $entryid = shift @{$self->param('path')};
-	if ($entryid =~ /^\d+$/) {
-		return $self->param(entry => Rubric::Entry->retrieve($entryid));
-	}
-	return;
+	$self->param(entry => Rubric::Entry->retrieve($self->next_path_part));
+	return $self;
 }
 
 =head2 link
@@ -302,8 +270,7 @@ URI or MD5 sum.
 
 sub link {
 	my ($self) = @_;
-	return $self->redirect_root("...no such link")
-		unless $self->get_link;
+	return $self->redirect_root("...no such link") unless $self->get_link;
 	$self->display_entries;
 }
 
@@ -318,14 +285,15 @@ parameter.
 sub get_link {
 	my ($self) = @_;
 	my %search;
-	#$search{md5sum} = $self->query->param('md5sum');
-	$search{uri}    = $self->query->param('uri') || $self->query->param('url');
+	$search{md5} = $self->query->param('md5');
+	$search{uri} = $self->query->param('uri') || $self->query->param('url');
 	for (qw(md5sum uri)) {
 		delete $search{$_} unless $search{$_};
 	}
 	return unless %search;
 	return unless my ($link) = Rubric::Link->search(\%search);
 	$self->param('link', $link);
+	return $self;
 }
 
 =head2 login
@@ -375,11 +343,12 @@ sub preferences {
 	return $self->redirect_root("permission denied")
 		unless $self->param('current_user');
 	
-	my %prefs  = $self->_get_prefs_form;
-	return $self->template("preferences") unless %prefs;
+	return $self->template("preferences")
+		unless my %prefs = $self->_get_prefs_form;
 
-	my %errors = $self->validate_prefs(\%prefs);
-	return $self->template("preferences", { %prefs, %errors } ) if %errors;
+	if (my %errors = $self->validate_prefs(\%prefs)) {
+		return $self->template("preferences", { %prefs, %errors } );
+	}
 
 	$self->update_user(\%prefs);
 }
@@ -499,22 +468,33 @@ sub validate_newuser_form {
 sub create_newuser {
 	my ($self, %newuser) = @_;
 
-	my $user = Rubric::User->create({
+	my %user = (
 		username => $newuser{username},
 		password => md5_hex($newuser{password_1}),
 		email    => $newuser{email},
-		verification_code => md5_hex("%newuser".time)
-	});
+	);
 
-	Rubric::Renderer->renderer('txt')->process(
-		'newuser_mail.txt',
-		{ user => $user, email_from => Rubric::Config->email_from },
-		\(my $message)
+	$user{verification_code} = md5_hex("%user".time)
+		unless Rubric::Config->skip_newuser_verification;
+
+	my $user = Rubric::User->create(\%user);
+
+	$self->send_verification_email_to($user)
+		unless Rubric::Config->skip_newuser_verification;
+
+	$self->template("account_created");
+}
+
+sub send_verification_email_to {
+	my ($self, $user) = @_;
+
+	my $message = Rubric::Renderer->process(
+		'newuser_mail',
+		'txt',
+		{ user => $user, email_from => Rubric::Config->email_from }
 	);
 
 	send SMTP => $message => Rubric::Config->smtp_server;
-
-	$self->template("account_created");
 }
 
 =head2 verify
@@ -543,20 +523,11 @@ sub verify {
 sub get_verification_code {
 	my ($self) = @_;
 
-	my $verification_code = shift @{$self->param('path')};
-	$self->param(verification_code => $verification_code || '');
+	$self->param(verification_code => $self->next_path_part);
 
 	return $self;
 }
 
-=head2 user
-
-This method will find the user requested (the name after "/user/" in the path)
-and note it in the request.  The request is then redispatched to the C<tag>
-method.
-
-=cut
-
 sub user {
 	my ($self) = @_;
 	$self->get_user->get_tags->display_entries;
@@ -565,54 +536,11 @@ sub user {
 sub get_user {
 	my ($self) = @_;
 
-	my $username = shift @{$self->param('path')};
-	if ($username =~ /^[\w\d.]+$/) {
-		$self->param(user => Rubric::User->retrieve($username) || '');
-	}
+	$self->param(user => Rubric::User->retrieve($self->next_path_part) || '');
 
 	return $self;
 }
 
-=head2 tag
-
-This method notes the requested tags (the words after /tag or /user/NAME in the
-path) and redispatches to C<display_entries>.
-
-=cut
-
-sub tag {
-	my ($self) = @_;
-
-	$self->get_tags->display_entries;
-}
-
-sub get_tags {
-	my ($self) = @_;
-
-	my $tags = shift @{$self->param('path')};
-
-	$self->param(tags => [ split /\+/, ($tags || '') ]);
-
-	return $self;
-}
-
-=head2 recent
-
-This method uses C<page_entries> and C<render_entries> to display the most
-recent entries for all users and tags.
-
-This should probably be what happens when C<display_entries> is called with no
-search criteria in place.
-
-=cut
-
-sub recent {
-	my ($self) = @_;
-
-	$self->param('recent_tags', Rubric::Entry->recent_tags_counted);
-	$self->display_entries;
-}
-
 =head2 display_entries
 
 This method searches (with Rubric::Entry) for entries matching the requested
@@ -658,8 +586,8 @@ entries representing the current page.  The following parameters are set:
 sub page_entries {
 	my ($self, $iterator) = @_;
 
-	my $first = $self->param('per_page') * ($self->param('page') - 1);
-	my $last  = ($self->param('per_page') * $self->param('page')) - 1;
+	my $first =  $self->param('per_page') * ($self->param('page')  - 1);
+	my $last  = ($self->param('per_page') *  $self->param('page')) - 1;
 	my $slice = $iterator->slice($first, $last);
 	$self->param('entries', $slice);
 	$self->param('count', $iterator->count);
@@ -679,17 +607,15 @@ C<page_entries>.
 =cut
 
 sub render_entries {
-	my ($self) = @_;
+	my ($self, $options) = @_;
+	$options ||= {};
 
 	$self->template('entries' => {
-		user    => $self->param('user'),
-		tags    => $self->param('tags'),
 		count   => $self->param('count'),
 		entries => $self->param('entries'),
 		pages   => $self->param('pages'),
+		%$options,
 		remove  => sub { [ grep { $_ ne $_[0] } @{$_[1]} ] },
-		has_link    => $self->param('has_link'),
-		has_body    => $self->param('has_body'),
 		long_form   => scalar $self->query->param('long_form'),
 		recent_tags => $self->param('recent_tags'),
 	});
@@ -751,7 +677,7 @@ sub post {
 
 	my $goto_uri = ($self->query->param('when_done') eq 'go_back')
 		? $entry{uri}
-		: Rubric::Config->uri_root() . "/user/" . $self->param('current_user');
+		: Rubric::WebApp::URI->entries({ user => $self->param('current_user') });
 	
 	return $self->redirect( $goto_uri, "Posted..." );
 }
@@ -808,9 +734,10 @@ sub delete {
 
 	$self->param('entry')->delete;
 
-	my $goto_uri = Rubric::WebApp::URI->user($self->param('current_user'));
-
-	return $self->redirect( $goto_uri, "Deleted..." );
+	return $self->redirect(
+		Rubric::WebApp::URI->entries({user=>$self->param('current_user')}),
+		"Deleted..."
+	);
 }
 
 =head2 doc
@@ -834,9 +761,8 @@ This gets the next part of the path and puts it in the C<doc_page> parameter.
 sub get_doc {
 	my ($self) = @_;
 
-	my $doc_page = shift @{$self->param('path')};
-	$self->param(doc_page => $doc_page)
-		if $doc_page =~ /^\w+$/;
+	my $doc_page = $self->next_path_part;
+	$self->param(doc_page => $doc_page) if $doc_page =~ /^\w+$/;
 
 	return $self;
 }
